"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Registry = void 0;
var _child_process = require("child_process");
var _debug = _interopRequireDefault(require("debug"));
var _fs = _interopRequireDefault(require("fs"));
var _path = _interopRequireDefault(require("path"));
var _config = require("@verdaccio/config");
var _core = require("@verdaccio/core");
var _utils = require("@verdaccio/utils");
var _request = require("./request");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const {
  writeFile
} = _fs.default.promises ? _fs.default.promises : require('fs/promises');
const buildAuthHeader = token => {
  return (0, _utils.buildToken)(_core.TOKEN_BEARER, token);
};
const debug = (0, _debug.default)('verdaccio:registry');
const defaultOptions = {
  domain: 'localhost',
  port: 4873,
  createUser: false,
  credentials: {
    user: 'foo',
    password: '12345'
  },
  debug: false
};
class Registry {
  authstr = null;
  token = null;
  constructor(configPath, options = {}) {
    const _options = {
      ...defaultOptions,
      ...options
    };
    this.configPath = configPath;
    this.createUser = _options.createUser;
    this.port = _options.port;
    this.domain = _options.domain;
    this.debug = _options.debug;
    this.credentials = _options.credentials;
  }
  static async fromConfigToPath(config) {
    debug(`fromConfigToPath`);
    const tempFolder = await _core.fileUtils.createTempFolder('registry-');
    debug(`tempFolder %o`, tempFolder);
    const yamlContent = (0, _config.fromJStoYAML)(config);
    const configPath = _path.default.join(tempFolder, 'registry.yaml');
    await writeFile(configPath, yamlContent);
    debug(`configPath %o`, configPath);
    return {
      tempFolder,
      configPath,
      yamlContent
    };
  }
  init(verdaccioPath) {
    return this._start(verdaccioPath);
  }
  getToken() {
    return this.token;
  }
  getAuthStr() {
    return this.authstr;
  }
  getPort() {
    return this.port;
  }
  getDomain() {
    return this.domain;
  }
  getRegistryUrl() {
    return `http://${this.getDomain()}:${this.getPort()}`;
  }
  _start(verdaccioPath = _path.default.join(__dirname, '../../bin/verdaccio')) {
    debug('_start %o', verdaccioPath);
    debug('port %o', this.port);
    return new Promise((resolve, reject) => {
      let childOptions = {
        silent: false
      };
      if (this.debug) {
        const debugPort = this.port + 5;
        debug('debug port %o', debugPort);
        childOptions = Object.assign({}, childOptions, {
          execArgv: [`--inspect=${debugPort}`],
          env: {
            DEBUG: process.env.DEBUG,
            VERDACCIO_SERVER: process.env.VERDACCIO_SERVER
          }
        });
      } else {
        childOptions = Object.assign({}, childOptions, {
          env: {
            DEBUG: process.env.DEBUG,
            VERDACCIO_SERVER: process.env.VERDACCIO_SERVER
          }
        });
      }
      const {
        configPath
      } = this;
      debug('configPath %s', configPath);
      debug('port %s', this.port);
      this.childFork = (0, _child_process.fork)(verdaccioPath, ['-c', configPath, '-l', String(this.port)], childOptions);
      this.childFork.on('message', async msg => {
        // verdaccio_started is a message that comes from verdaccio in debug mode that
        // notify has been started
        try {
          if ('verdaccio_started' in msg) {
            const server = new _request.ServerQuery(`http://${this.domain}:` + this.port);
            if (this.createUser) {
              var _user$response;
              const user = await server.createUser(this.credentials.user, this.credentials.password);
              user.status(_core.HTTP_STATUS.CREATED).body_ok(new RegExp(this.credentials.user));
              // @ts-ignore
              this.token = user === null || user === void 0 ? void 0 : (_user$response = user.response) === null || _user$response === void 0 ? void 0 : _user$response.body.token;
              this.authstr = buildAuthHeader(this.token);
            }
            return resolve(this.childFork);
          }
        } catch (e) {
          // eslint-disable-next-line no-console
          console.error(e);
          // eslint-disable-next-line prefer-promise-reject-errors
          return reject([e, this]);
        }
      });
      this.childFork.on('error', err => {
        debug('error  %s', err);
        // eslint-disable-next-line prefer-promise-reject-errors
        reject([err, this]);
      });
      this.childFork.on('disconnect', err => {
        // eslint-disable-next-line prefer-promise-reject-errors
        reject([err, this]);
      });
      this.childFork.on('exit', err => {
        // eslint-disable-next-line prefer-promise-reject-errors
        reject([err, this]);
      });
    });
  }
  stop() {
    return this.childFork.kill('SIGINT');
  }
}
exports.Registry = Registry;
//# sourceMappingURL=registry.js.map